----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch
--
-- Module Name: gt_common - rtl
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2020.2
-- Description: Aurora GTP common
--
-- Last update: 2020-09-28
--
---------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;

entity gt_common is

    generic (
        C_SIM_GTRESET_SPEEDUP : string := "false");

    port (
        -- Common Block - Ports
        GT0GTRefClk0xCI      : in  std_logic;
        GT0PLL0LockxSO       : out std_logic;
        GT0PLL1LockxSO       : out std_logic;
        GT0PLL0LockDetClkxCI : in  std_logic;
        GT0PLL0RefClkLostxSO : out std_logic;
        -- Common Ports
        GT0PLL0ClkxCO        : out std_logic;
        GT0PLL1ClkxCO        : out std_logic;
        GT0PLL0RefClkxCO     : out std_logic;
        GT0PLL1RefClkxCO     : out std_logic;
        GT0PLL0ResetxRI      : in  std_logic);

end entity gt_common;

architecture rtl of gt_common is

    -- Constants
    ---------------------------------------------------------------------------
    constant C_PLL0_FBDIV_IN      : integer := 2;
    constant C_PLL1_FBDIV_IN      : integer := 4;
    constant C_PLL0_FBDIV_45_IN   : integer := 5;
    constant C_PLL1_FBDIV_45_IN   : integer := 5;
    constant C_PLL0_REFCLK_DIV_IN : integer := 1;
    constant C_PLL1_REFCLK_DIV_IN : integer := 1;

    -- Signals
    ---------------------------------------------------------------------------
    signal TiedToGroundxS         : std_logic                      := '0';
    signal TiedToGroundxD         : std_logic_vector(63 downto 0)  := (others => '0');
    signal TiedToVCCxS            : std_logic                      := '1';
    signal CPLLPDQuad0WaitxD      : std_logic_vector(95 downto 0)  := x"FFFFFFFFFFFFFFFFFFFFFFFF";
    signal CPLLResetQuad0WaitxD   : std_logic_vector(127 downto 0) := x"000000000000000000000000000000FF";
    signal CPLLPDOvrdQuad0xS      : std_logic                      := '0';
    signal CPLLPDResetOvrdQuad0xR : std_logic                      := '0';
    signal CPLLResetQuad0xR       : std_logic                      := '0';

    -- Attributes
    attribute equivalent_register_removal                         : string;
    attribute equivalent_register_removal of CPLLPDQuad0WaitxD    : signal is "no";
    attribute equivalent_register_removal of CPLLResetQuad0WaitxD : signal is "no";

begin  -- architecture rtl

    -- Asynchronous statements

    TiedToGroundSxAS        : TiedToGroundxS         <= '0';
    TiedToGroundDxAS        : TiedToGroundxD         <= (others => '0');
    TiedToVCCxAS            : TiedToVCCxS            <= '1';
    CPLLResetQuad0xAS       : CPLLResetQuad0xR       <= CPLLPDResetOvrdQuad0xR or GT0PLL0ResetxRI;
    CPLLPDOvrdQuad0xAS      : CPLLPDOvrdQuad0xS      <= CPLLPDQuad0WaitxD(95);
    CPLLPDResetOvrdQuad0xAS : CPLLPDResetOvrdQuad0xR <= CPLLResetQuad0WaitxD(127);

    -- GTPE2 Common (PLL0 and PLL1)
    ---------------------------------------------------------------------------
    GTPE2CommonxI : GTPE2_COMMON
        generic map (
            -- Simulation attributes
            SIM_RESET_SPEEDUP  => C_SIM_GTRESET_SPEEDUP,
            SIM_PLL0REFCLK_SEL => ("001"),
            SIM_PLL1REFCLK_SEL => ("001"),
            SIM_VERSION        => ("2.0"),
            PLL0_FBDIV         => C_PLL0_FBDIV_IN,
            PLL0_FBDIV_45      => C_PLL0_FBDIV_45_IN,
            PLL0_REFCLK_DIV    => C_PLL0_REFCLK_DIV_IN,
            PLL1_FBDIV         => C_PLL1_FBDIV_IN,
            PLL1_FBDIV_45      => C_PLL1_FBDIV_45_IN,
            PLL1_REFCLK_DIV    => C_PLL1_REFCLK_DIV_IN,
            -- Common Block Attributes
            BIAS_CFG           => (x"0000000000050001"),
            COMMON_CFG         => (x"00000000"),
            -- PLL Attributes
            PLL0_CFG           => (x"01F03DC"),
            PLL0_DMON_CFG      => ('0'),
            PLL0_INIT_CFG      => (x"00001E"),
            PLL0_LOCK_CFG      => (x"1E8"),
            PLL1_CFG           => (x"01F03DC"),
            PLL1_DMON_CFG      => ('0'),
            PLL1_INIT_CFG      => (x"00001E"),
            PLL1_LOCK_CFG      => (x"1E8"),
            PLL_CLKOUT_CFG     => (x"00"),
            -- Reserved Attributes
            RSVD_ATTR0         => (x"0000"),
            RSVD_ATTR1         => (x"0000"))
        port map (
            DMONITOROUT       => open,
            -- Common Block - Dynamic Reconfiguration Port (DRP)
            DRPADDR           => TiedToGroundxD(7 downto 0),
            DRPCLK            => TiedToGroundxS,
            DRPDI             => TiedToGroundxD(15 downto 0),
            DRPDO             => open,
            DRPEN             => TiedToGroundxS,
            DRPRDY            => open,
            DRPWE             => TiedToGroundxS,
            -- Common Block - Ports
            BGRCALOVRDENB     => TiedToVCCxS,
            GTEASTREFCLK0     => TiedToGroundxS,
            GTEASTREFCLK1     => TiedToGroundxS,
            GTGREFCLK0        => TiedToGroundxS,
            GTGREFCLK1        => TiedToGroundxS,
            GTREFCLK0         => GT0GTRefClk0xCI,
            GTREFCLK1         => TiedToGroundxS,
            GTWESTREFCLK0     => TiedToGroundxS,
            GTWESTREFCLK1     => TiedToGroundxS,
            PLL0FBCLKLOST     => open,
            PLL0LOCK          => GT0PLL0LockxSO,
            PLL0LOCKDETCLK    => GT0PLL0LockDetClkxCI,
            PLL0LOCKEN        => TiedToVCCxS,
            PLL0OUTCLK        => GT0PLL0ClkxCO,
            PLL0OUTREFCLK     => GT0PLL0RefClkxCO,
            PLL0PD            => CPLLPDOvrdQuad0xS,
            PLL0REFCLKLOST    => GT0PLL0RefClkLostxSO,
            PLL0REFCLKSEL     => "001",
            PLL0RESET         => CPLLResetQuad0xR,
            PLL1FBCLKLOST     => open,
            PLL1LOCK          => GT0PLL1LockxSO,
            PLL1LOCKDETCLK    => TiedToGroundxS,
            PLL1LOCKEN        => TiedToVCCxS,
            PLL1OUTCLK        => GT0PLL1ClkxCO,
            PLL1OUTREFCLK     => GT0PLL1RefClkxCO,
            PLL1PD            => '1',
            PLL1REFCLKLOST    => open,
            PLL1REFCLKSEL     => "001",
            PLL1RESET         => CPLLPDResetOvrdQuad0xR,
            PLLRSVD1          => "0000000000000000",
            PLLRSVD2          => "00000",
            PMARSVDOUT        => open,
            REFCLKOUTMONITOR0 => open,
            REFCLKOUTMONITOR1 => open,
            -- Common Block Ports
            BGBYPASSB         => TiedToVCCxS,
            BGMONITORENB      => TiedToVCCxS,
            BGPDB             => TiedToVCCxS,
            BGRCALOVRD        => "11111",
            PMARSVD           => "00000000",
            RCALENB           => TiedToVCCxS);

    -- Synchronous statements

    -- Wait process for PD and Reset signals
    ---------------------------------------------------------------------------
    CPLLPDAndResetWaitxP : process (GT0GTRefClk0xCI) is
    begin  -- process CPLLPDAndResetWaitxP
        if rising_edge(GT0GTRefClk0xCI) then
            CPLLPDQuad0WaitxD    <= CPLLPDQuad0WaitxD(94 downto 0) & '0';
            CPLLResetQuad0WaitxD <= CPLLResetQuad0WaitxD(126 downto 0) & '0';
        end if;
    end process CPLLPDAndResetWaitxP;

end architecture rtl;
